"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const node_path_1 = __importDefault(require("node:path"));
const glob_1 = __importDefault(require("glob"));
const graphql_1 = require("graphql");
const amplify_cli_core_1 = require("amplify-cli-core");
const extension_1 = require("../extension");
const feature_flags_1 = require("../feature-flags");
const run = async (context) => {
    var _a;
    // Initialize feature flags
    if (!amplify_cli_core_1.FeatureFlags.isInitialized()) {
        const contextEnvironmentProvider = new amplify_cli_core_1.CLIContextEnvironmentProvider({
            getEnvInfo: context.amplify.getEnvInfo,
        });
        const projectPath = (_a = amplify_cli_core_1.pathManager.findProjectRoot()) !== null && _a !== void 0 ? _a : process.cwd();
        const useNewDefaults = !amplify_cli_core_1.stateManager.projectConfigExists(projectPath);
        await amplify_cli_core_1.FeatureFlags.initialize(contextEnvironmentProvider, useNewDefaults);
    }
    const allApiResources = await context.amplify.getResourceStatus("api");
    const apiResource = allApiResources.allResources.find((resource) => resource.service === "AppSync" &&
        resource.providerPlugin === "awscloudformation");
    if (!apiResource) {
        context.print.info("No AppSync API configured. Please add an API");
        return;
    }
    const backendPath = await context.amplify.pathManager.getBackendDirPath();
    const apiResourcePath = node_path_1.default.join(backendPath, "api", apiResource.resourceName);
    const directiveDefinitions = await context.amplify.executeProviderUtils(context, "awscloudformation", "getTransformerDirectives", {
        resourceDir: apiResourcePath,
    });
    const schemaContent = loadSchema(apiResourcePath);
    const schema = (0, graphql_1.parse)(schemaContent);
    const modelFolder = "lib/models";
    const generatedOutput = await (0, extension_1.generateExtensions)({
        schema,
        directives: directiveDefinitions,
        baseOutputDir: modelFolder,
        generateIndexRules: (0, feature_flags_1.readFeatureFlag)(context, "codegen.generateIndexRules"),
        emitAuthProvider: (0, feature_flags_1.readFeatureFlag)(context, "codegen.emitAuthProvider"),
        transformerVersion: (0, feature_flags_1.readNumericFeatureFlag)(context, "graphQLTransformer.transformerVersion"),
        respectPrimaryKeyAttributesOnConnectionField: (0, feature_flags_1.readFeatureFlag)(context, "graphQLTransformer.respectPrimaryKeyAttributesOnConnectionField"),
        generateModelsForLazyLoadAndCustomSelectionSet: (0, feature_flags_1.readFeatureFlag)(context, "codegen.generateModelsForLazyLoadAndCustomSelectionSet"),
        addTimestampFields: (0, feature_flags_1.readFeatureFlag)(context, "codegen.addTimestampFields"),
        handleListNullabilityTransparently: (0, feature_flags_1.readFeatureFlag)(context, "codegen.handleListNullabilityTransparently"),
    });
    Object.entries(generatedOutput).map(([filename, content]) => {
        fs_extra_1.default.ensureFileSync(filename);
        fs_extra_1.default.writeFileSync(filename, content);
    });
    context.print.info(`Successfully generated extensions. Generated models can be found in ${modelFolder}`);
};
exports.run = run;
function loadSchema(apiResourcePath) {
    const schemaFilePath = node_path_1.default.join(apiResourcePath, "schema.graphql");
    const schemaDirectory = node_path_1.default.join(apiResourcePath, "schema");
    if (fs_extra_1.default.pathExistsSync(schemaFilePath)) {
        return fs_extra_1.default.readFileSync(schemaFilePath, "utf8");
    }
    if (fs_extra_1.default.pathExistsSync(schemaDirectory) &&
        fs_extra_1.default.lstatSync(schemaDirectory).isDirectory()) {
        // search recursively for graphql schema files inside `schema` directory
        const schemas = glob_1.default.sync([node_path_1.default.join(schemaDirectory, "**/*.graphql")]);
        return schemas.map((file) => fs_extra_1.default.readFileSync(file, "utf8")).join("\n");
    }
    throw new Error("Could not load the schema");
}
//# sourceMappingURL=codegen.js.map